from scipy.constants import c, e
from iminuit import Minuit
from iminuit.cost import LeastSquares
import matplotlib.pyplot as plt 
import numpy as np
from ..library import array


frequency = [int(c/632E-9), int(c/628E-9), int(c/519E-9), int(c/462E-9)]


def model(frequency, m, c):
    return (m / e) * frequency - c


def plot(frequency, voltage, error, filename="plot.png"):
    fontsize = 14
    
    plt.figure(figsize=(8, 6), dpi=300)
    plt.errorbar(frequency, voltage, yerr=error, fmt='o', label='Data', color='black')
    plt.xlabel("Frequency $(Hz)$", fontsize=fontsize)
    plt.ylabel("Voltage $(V)$", fontsize=fontsize)
    plt.grid()
    plt.savefig(filename)
    plt.clf()
    
    
def fit(frequency, voltage, error, m=1, c=0, filename="fit.png", eV=False):
    fontsize = 14
    
    frequency = np.array(frequency)
    voltage = np.array(voltage)
    error = np.array(error)
    
    ls = LeastSquares(frequency, voltage, error, model)
    m = Minuit(ls, m=m, c=c)
    m.migrad()
    m.hesse()
    
    fit_info = []
    parameters = ["Plank's Constant".ljust(24), "Heat Energy Contribution".ljust(24)]
    units = ["Js", "eV"]
    for p, v, e, u in zip(parameters, m.values, m.errors, units):
        fit_info.append(f"{p} = ${v:.2E} \\pm {e:.1E}$ {u}")

    
    plt.figure(figsize=(8, 6), dpi=300)
    plt.rcParams.update({"font.family": "monospace"})
    
    plt.errorbar(frequency, voltage, yerr=error, fmt='o', label='Data', color='black')
    plt.plot(frequency,  model(frequency, *m._values), label='Fit', color='grey', linestyle='--')
    
    if eV:
        plt.ylabel("Energy $(eV)$", fontsize=fontsize)
    else:
        plt.ylabel("Voltage $(V)$", fontsize=fontsize)
        
    plt.xlabel("Frequency $(Hz)$", fontsize=fontsize)
    plt.legend(title="\n".join(fit_info), fontsize=fontsize, title_fontsize=fontsize, frameon=False)
    
    plt.grid()
    plt.tick_params(axis='both', which='major', labelsize=fontsize, direction='in')
    plt.savefig(filename)
    plt.clf()
    
    


import numpy as np
import time
import logging
import ncuphy.highschool.ncugrovepi as ncugrovepi


class AnalogPort:
    def __init__(self, pin: int) -> None:
        self.__available_ports = [0, 1, 2]
        self.__bits = 10
        self.__ref_voltage = 5

        assert pin in self.__available_ports

        self.pin = pin

        ncugrovepi.pinMode(self.pin, "INPUT")

    def __raw(self) -> int:
        while True:
            try:
                value = ncugrovepi.analogRead(self.pin)
                assert value <= 2 ** self.__bits
                return value
            except:
                logging.warning("Analog read raw directly from adc failed. Retrying...")
                continue

    @property
    def value(self) -> int:
        return self.__raw()

    @property
    def voltage(self) -> float:
        return self.__raw() * self.__ref_voltage / 2 ** self.__bits

    def sample(self, N: int, mode='voltage') -> tuple[float, float]:
        assert mode in ['voltage', 'value']
        assert isinstance(N, int)
        assert N > 0

        data = np.empty(N)

        if mode == 'voltage':
            for i in range(N):
                data[i] = self.voltage
        elif mode == 'value':
            for i in range(N):
                data[i] = self.value

        return np.mean(data), np.std(data)


class DigitalPort:
    def __init__(self, pin: int, mode: str) -> None:
        self.__available_ports = [3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15]
        self.__modes = ['OUTPUT', 'INPUT']

        assert pin in self.__available_ports
        assert mode in self.__modes

        self.pin = pin

        ncugrovepi.pinMode(self.pin, mode)

    def __write(self, value: int) -> None:
        assert value in [0, 1]
        ncugrovepi.digitalWrite(self.pin, value)

    def high(self) -> None:
        self.__write(1)

    def low(self) -> None:
        self.__write(0)

    def pulse(self, duration: float) -> None:
        assert isinstance(duration, (int, float))
        assert duration > 0

        ncugrovepi.digitalWrite(self.pin, 1)
        start = time.time()

        while time.time() - start < duration:
                pass

        ncugrovepi.digitalWrite(self.pin, 0)
        start = time.time()

        while time.time() - start < duration:
                pass


class StepperMotor:
    def __init__(self, port: int) -> None:
        GND = "GND"
        VCC = "VCC"
        DIR = "DIR"
        PUL = "PUL"

        self.__available_ports = [0, 2, 3, 4, 5, 6, 7, 8]

        self.__port_mapping = {0: (GND, VCC, 15, 14),
                               2: (GND, VCC, 3, 2),
                               3: (GND, VCC, 4, 3),
                               4: (GND, VCC, 5, 4),
                               5: (GND, VCC, 6, 5),
                               6: (GND, VCC, 7, 6),
                               7: (GND, VCC, 8, 7),
                               8: (GND, VCC, 9, 8)}

        self.__default_port_config = (GND, VCC, DIR, PUL)

        assert port in self.__available_ports

        self.port = port
        self.gnd, self.vcc, self.dir, self.pul = self.__port_mapping[port]

        self.dir_port = DigitalPort(self.dir, "OUTPUT")
        self.pul_port = DigitalPort(self.pul, "OUTPUT")

    def step(self, steps: int) -> None:
        assert isinstance(steps, int)

        delay = 0.00075

        if steps < 0:
            self.dir_port.high()
        else:
            self.dir_port.low()

        for i in range(abs(steps)):
            start = time.time()

            self.pul_port.high()
            while time.time() - start < delay:
                pass

            start = time.time()

            self.pul_port.low()
            while time.time() - start < delay:
                pass

    def move(self, microsteps, displacement):
        assert isinstance(microsteps, int)
        assert isinstance(displacement, (int, float))

        steps = int(microsteps * displacement / 1.25)
        self.step(steps)


class OPT101:
    def __init__(self, port: int):
        GND = "GND"
        VCC = "VCC"
        ADC = "ADC"
        EMPTY = "EMPTY"

        self.__available_ports = [0, 1, 2]

        self.__port_mapping = {0: (GND, VCC, 1, 0),
                               1: (GND, VCC, 2, 1),
                               2: (GND, VCC, 3, 2)}

        self.__default_port_config = (GND, VCC, ADC, EMPTY)

        assert port in self.__available_ports

        self.port = port

        self.gnd, self.vcc, self.adc, self.empty = self.__port_mapping[port]
        self.adc_port = AnalogPort(self.adc)

    @property
    def voltage(self):
        return self.adc_port.voltage

    @property
    def value(self):
        return self.adc_port.value

    def sample(self, N: int, mode='voltage') -> tuple[float, float]:
        assert mode in ['voltage', 'value']
        assert isinstance(N, int)
        assert N > 0

        return self.adc_port.sample(N, mode)


class PlanksConstant:
    def __init__(self, port: int):
        GND = "GND"
        VCC = "VCC"
        CH1 = "CH1"
        CH2 = "CH2"
        
        self.__available_ports = [0, 1, 2]
        
        self.__port_mapping = {0: (GND, VCC, 0, 1),
                                 1: (GND, VCC, 2, 1),
                                 2: (GND, VCC, 3, 1)}
        
        self.__default_port_config = (GND, VCC, CH1, CH2)
        
        assert port in self.__available_ports
        
        self.port = port
        
        self.gnd, self.vcc, self.ch1, self.ch2 = self.__port_mapping[port]
        self.ch1_port = AnalogPort(self.ch1)
        self.ch2_port = AnalogPort(self.ch2)
        
    @property
    def voltage(self):
        return self.ch1_port.voltage, self.ch2_port.voltage
    
    @property
    def value(self):
        return self.ch1_port.value, self.ch2_port.value
    
    def sample(self, N: int, mode='voltage') -> tuple[float, float]:
        assert mode in ['voltage', 'value']
        assert isinstance(N, int)
        assert N > 0
        
        return self.ch1_port.sample(N, mode), self.ch2_port.sample(N, mode)

def array(item):
    return item if isinstance(item, np.ndarray) else np.array(item)



from smbus2 import SMBus
import time


class MPU6050:
    """
    This class represents the MPU6050 sensor and provides methods to read accelerometer and gyroscope data.

    Args:
        address (int): I2C address of the MPU6050 sensor (default is 0x68) (0x69 when AD0 is set to high).
        accel_fsr (int): Accelerometer full scale range (default is 2g).
        gyro_fsr (int): Gyroscope full scale range (default is 250 degrees/second).
        sample_rate_divider (int): Sample rate divider (default is 1).
        digital_lowpass_level (int): Digital low pass filter configuration (default is 0).
        ext_sync_set (int): External sync setting (default is 0).

    Raises:
        AssertionError: If any of the input arguments are invalid.

    Methods:
        get_accel_data(): Get the accelerometer data with unit in g.
        get_gyro_data(): Get the gyroscope data with unit of degrees/second.
        
    Attributes:
        PWR_MGMT_1 (int): Power Management register address.
        
        SMPLRT_DIV (int): Sampling Rate Division register address.
        
        CONFIG (int): Frame Synchronization, Digital Low Pass Filter, and Sampling Rate register address.
        GYRO_CONFIG (int): Gyroscope Full Scale Range register address.
        ACCEL_CONFIG (int): Accelerometer Full Scale Range register address.
        
        INT_ENABLE (int): Interrupt Enable register address.
        
        ACCEL_XOUT_H (int): Accelerometer Readings - High Bytes register address for X-axis.
        ACCEL_YOUT_H (int): Accelerometer Readings - High Bytes register address for Y-axis.
        ACCEL_ZOUT_H (int): Accelerometer Readings - High Bytes register address for Z-axis.
        
        GYRO_XOUT_H (int): Gyroscope Readings - High Bytes register address for X-axis.
        GYRO_YOUT_H (int): Gyroscope Readings - High Bytes register address for Y-axis.
        GYRO_ZOUT_H (int): Gyroscope Readings - High Bytes register address for Z-axis.
        
        DLPF_CONFIGS (dict): Dictionary containing Digital Low Pass Filter and Sample Rate configurations.
        GYRO_CONFIGS (dict): Dictionary containing Gyroscope Full Scale Range configurations.
        ACCEL_CONFIGS (dict): Dictionary containing Accelerometer Full Scale Range configurations.
    """

    # Rest of the code...
class MPU6050:
    # Power Management
    PWR_MGMT_1 = 0x6B
    
    # Sampling Rate Division
    SMPLRT_DIV = 0x19
    
    # Frame Synchronization, Digital Low Pass Filter, Sampling Rate
    CONFIG = 0x1A
    
    # Gyroscope Full Scale Range
    GYRO_CONFIG = 0x1B
    
    # Accelerometer Full Scale Range
    ACCEL_CONFIG = 0x1C
    
    # Interrupt Enable
    INT_ENABLE = 0x38
    
    # Accelerometer Readings - High Bytes
    ACCEL_XOUT_H = 0x3B
    ACCEL_YOUT_H = 0x3D
    ACCEL_ZOUT_H = 0x3F
    
    # Gyroscope Readings - High Bytes
    GYRO_XOUT_H = 0x43
    GYRO_YOUT_H = 0x45
    GYRO_ZOUT_H = 0x47

    # Digital Low Pass Filter and Sample Rate Config, units (Hz, Hz, Hz)
    DLPF_CONFIGS = {
        0: {'gyro_bw': 256, 'accel_bw': 260, 'base_rate': 8000},   # DLPF_CFG = 0
        1: {'gyro_bw': 188, 'accel_bw': 184, 'base_rate': 1000},   # DLPF_CFG = 1
        2: {'gyro_bw': 98,  'accel_bw': 94,  'base_rate': 1000},   # DLPF_CFG = 2
        3: {'gyro_bw': 42,  'accel_bw': 44,  'base_rate': 1000},   # DLPF_CFG = 3
        4: {'gyro_bw': 20,  'accel_bw': 21,  'base_rate': 1000},   # DLPF_CFG = 4
        5: {'gyro_bw': 10,  'accel_bw': 10,  'base_rate': 1000},   # DLPF_CFG = 5
        6: {'gyro_bw': 5,   'accel_bw': 5,   'base_rate': 1000},   # DLPF_CFG = 6
        7: {'gyro_bw': None, 'accel_bw': None, 'base_rate': None}  # Reserved
    }
    
    GYRO_CONFIGS = {
        250: 0x00,  # +/- 250 degrees/second
        500: 0x08,  # +/- 500 degrees/second
        1000: 0x10, # +/- 1000 degrees/second
        2000: 0x18  # +/- 2000 degrees/second
    }
    
    ACCEL_CONFIGS = {
        2: 0x00,   # +/- 2g
        4: 0x08,   # +/- 4g
        8: 0x10,   # +/- 8g
        16: 0x18   # +/- 16g
    }

    def __init__(self, address=0x68, accel_fsr=2, gyro_fsr=250, sample_rate_divider=1, digital_lowpass_level=0, ext_sync_set=0):
        valid_accel_fsr = [2, 4, 8, 16]
        assert accel_fsr in valid_accel_fsr, f"accel_fsr {accel_fsr} is not in the list of valid full scale ranges {valid_accel_fsr}"
        
        valid_gyro_fsr = [250, 500, 1000, 2000]
        assert gyro_fsr in valid_gyro_fsr, f"gyro_fsr {gyro_fsr} is not in the list of valid full scale ranges {valid_gyro_fsr}"
        
        valid_dividers = list(range(1, 256))
        assert sample_rate_divider in valid_dividers, f"sample_rate_divider {sample_rate_divider} is not in the list of valid dividers {valid_dividers}"
        
        valid_dlpf_configs = [0, 1, 2, 3, 4, 5, 6]
        assert digital_lowpass_level in valid_dlpf_configs, f"digital_lowpass_config {digital_lowpass_level} is not in the list of valid configurations {valid_dlpf_configs}"

        valid_ext_sync = [0, 1, 2, 3, 4, 5, 6, 7]
        assert ext_sync_set in valid_ext_sync, f"ext_sync_set {ext_sync_set} is not in the list of valid external sync settings {valid_ext_sync}"
        
        self.address = address
        self.bus = SMBus(1) 
        self.accel_fsr = accel_fsr
        self.gyro_fsr = gyro_fsr
        self.sample_rate_divider = sample_rate_divider
        self.digital_lowpass_level = digital_lowpass_level
        self.ext_sync_set = ext_sync_set

        # Wake up the MPU6050
        self.bus.write_byte_data(self.address, MPU6050.PWR_MGMT_1, 0x00)
        time.sleep(0.1)

        # Set the DLPF and EXT_SYNC_SET configuration
        config_value = (self.ext_sync_set << 3) | self.digital_lowpass_level
        self.bus.write_byte_data(self.address, MPU6050.CONFIG, config_value)

        # Set the sample rate divider
        base_rate = MPU6050.DLPF_CONFIGS[self.digital_lowpass_level]['base_rate']
        if base_rate:
            smplrt_div = self.sample_rate_divider - 1
            self.bus.write_byte_data(self.address, MPU6050.SMPLRT_DIV, smplrt_div)
        else:
            raise ValueError(f"Invalid base rate for DLPF configuration {self.digital_lowpass_level}")

        # Set accelerometer full scale range
        self.bus.write_byte_data(self.address, MPU6050.ACCEL_CONFIG, MPU6050.ACCEL_CONFIGS[self.accel_fsr])

        # Set gyroscope full scale range
        self.bus.write_byte_data(self.address, MPU6050.GYRO_CONFIG, MPU6050.GYRO_CONFIGS[self.gyro_fsr])

        # Enable interrupts
        self.bus.write_byte_data(self.address, MPU6050.INT_ENABLE, 0x01)

        # Print the sample rate
        print(f"Sample rate: {base_rate / self.sample_rate_divider:.2f} Hz")
         
    def _read_raw_data(self, addr):
        # Accelero and Gyro values are 16-bit
        high = self.bus.read_byte_data(self.address, addr)
        low = self.bus.read_byte_data(self.address, addr + 1)
        value = (high << 8) | low
        
        # Convert to signed value
        if value > int(2 ** 15):
            value -= 2 ** 16
        return int(value)

    def get_accel_data(self):
        """
        Get the accelerometer data from the MPU6050 sensor with unit in g.

        Returns:
            dict: A dictionary containing the accelerometer data with keys 'x', 'y', and 'z'.
        """
        accel_x = self._read_raw_data(MPU6050.ACCEL_XOUT_H) * (self.accel_fsr / 2 ** 15)
        accel_y = self._read_raw_data(MPU6050.ACCEL_YOUT_H) * (self.accel_fsr / 2 ** 15)
        accel_z = self._read_raw_data(MPU6050.ACCEL_ZOUT_H) * (self.accel_fsr / 2 ** 15)
        return {'x': accel_x, 'y': accel_y, 'z': accel_z}

    def get_gyro_data(self):
        """
        Get the gyroscope data from the MPU6050 sensor with unit of degrees/second.

        Returns:
            dict: A dictionary containing the gyroscope data with keys 'x', 'y', and 'z'.
        """
        gyro_x = self._read_raw_data(MPU6050.GYRO_XOUT_H) * (self.gyro_fsr / 2 ** 15)
        gyro_y = self._read_raw_data(MPU6050.GYRO_YOUT_H) * (self.gyro_fsr / 2 ** 15)
        gyro_z = self._read_raw_data(MPU6050.GYRO_ZOUT_H) * (self.gyro_fsr / 2 ** 15)
        return {'x': gyro_x, 'y': gyro_y, 'z': gyro_z}


if __name__ == "__main__":
    mpu = MPU6050(accel_fsr=2, gyro_fsr=500, sample_rate_divider=1, digital_lowpass_level=6, ext_sync_set=0)  # Example initialization

    while True:
        accel_data = mpu.get_accel_data()
        gyro_data = mpu.get_gyro_data()

        print(f"Accelerometer: {accel_data}")
        print(f"Gyroscope: {gyro_data}\n")

        time.sleep(1)



__list__ = sorted(['StepperMotor', "ADS1115", "MPU9250", "MPU6050"])

__which__ = input(f"Which module do you want to import? \n{__list__}\nEnter the module name: ")

assert __which__ in __list__, f"Module {__which__} not found in {__list__}"


__stepper_motor_example = """
from ncuphy.experiment import StepperMotor
import time

# display log, not necessary ---------------------------------------------

import logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# -----------------------------------------------------------------------------

# define the pins
pul = 26
dir = 19
ena = 13

# define the delay between pulses
pulse_delay = 0.001

# create the stepper motor object
motor = StepperMotor(pul, dir, ena, pulse_delay)



# move the stepper motor by 1000 steps
motor.step(1000)
time.sleep(2)



# move the stepper motor in the other direction by 1500 steps
motor.step(-1800)
time.sleep(2)



# move the stepper motor to the home position
motor.home()
time.sleep(2)



# move the stepper motor again by 100 steps
motor.step(1200)
time.sleep(2)



# get the current position
position = motor.position
print(f"Current position: {position}")



# set the current position as the home position
motor.sethome()
"""


if __which__ == "StepperMotor":
    with open('Example_StepperMotor.py', 'w') as f:
        f.write(__stepper_motor_example)
else:
    print("Example not found.")



"""
## License

The MIT License (MIT)

GrovePi for the Raspberry Pi: an open source platform for connecting Grove Sensors to the Raspberry Pi.
Copyright (C) 2017  Dexter Industries

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import smbus2 as smbus
import RPi.GPIO as GPIO


rev = GPIO.RPI_REVISION
if rev == 2 or rev == 3:
    bus = smbus.SMBus(1)
else:
    bus = smbus.SMBus(0)

# I2C Address of Arduino
address = 0x04

# Command Format
# digitalRead() command format header
dRead_cmd = [1]
# digitalWrite() command format header
dWrite_cmd = [2]
# analogRead() command format header
aRead_cmd = [3]
# analogWrite() command format header
aWrite_cmd = [4]
# pinMode() command format header
pMode_cmd = [5]

unused = 0
retries = 10


# Write I2C block
def write_i2c_block(address, block):
	for i in range(retries):
		try:
			return bus.write_i2c_block_data(address, 1, block)
		except IOError:
			if debug:
				print ("IOError")
	return -1

# Read I2C byte
def read_i2c_byte(address):
	for i in range(retries):
		try:
			return bus.read_byte(address)
		except IOError:
			if debug:
				print ("IOError")
	return -1


# Read I2C block
def read_i2c_block(address):
	for i in range(retries):
		try:
			return bus.read_i2c_block_data(address, 1, 10)
		except IOError:
			if debug:
				print ("IOError")
	return -1

# Arduino Digital Read
def digitalRead(pin):
	write_i2c_block(address, dRead_cmd + [pin, unused, unused])
	# time.sleep(.1)
	n = read_i2c_byte(address)
	return n

# Arduino Digital Write
def digitalWrite(pin, value):
	write_i2c_block(address, dWrite_cmd + [pin, value, unused])
	return 1


# Setting Up Pin mode on Arduino
def pinMode(pin, mode):
	if mode == "OUTPUT":
		write_i2c_block(address, pMode_cmd + [pin, 1, unused])
	elif mode == "INPUT":
		write_i2c_block(address, pMode_cmd + [pin, 0, unused])
	return 1


# Read analog value from Pin
def analogRead(pin):
	write_i2c_block(address, aRead_cmd + [pin, unused, unused])
	read_i2c_byte(address)
	number = read_i2c_block(address)
	return number[1] * 256 + number[2]



import time
import logging
import RPi.GPIO as GPIO


def sleep(t):
    start = time.perf_counter()
    while time.perf_counter() - start < t:
        pass
    

class StepperMotor:
    """
    Represents a stepper motor.

    Args:
        pul (int): The GPIO pin number for the PUL (pulse) signal.
        dir (int): The GPIO pin number for the DIR (direction) signal.
        ena (int): The GPIO pin number for the ENA (enable) signal.
        step_delay (float or int): The delay between steps in seconds.

    Attributes:
        pul (int): The GPIO pin number for the PUL (pulse) signal.
        dir (int): The GPIO pin number for the DIR (direction) signal.
        ena (int): The GPIO pin number for the ENA (enable) signal.
        step_delay (float or int): The delay between steps in seconds.
        position (int): The current position of the stepper motor.

    Methods:
        step(steps: int): Moves the stepper motor by the specified number of steps.
        
        sethome(): Sets the current position as the home position.
        home(): Moves the stepper motor to the home position.
        
        hold(): Holds the stepper motor in its current position.
        release(): Releases the stepper motor.
    """
    def __init__(self, pul, dir, ena, step_delay):
        """
        Initializes a StepperMotor instance.

        Args:
            pul (int): The GPIO pin number for the PUL (pulse) signal.
            dir (int): The GPIO pin number for the DIR (direction) signal.
            ena (int): The GPIO pin number for the ENA (enable) signal.
            step_delay (float or int): The delay between steps in seconds.
        """
        gpiomode = GPIO.getmode()
        
        if gpiomode is None:
            GPIO.setmode(GPIO.BCM)
            
        logging.info(f"GPIO mode: {GPIO.getmode()}")
        logging.info(f"Step Delay: {step_delay} s")
        
        assert type(pul) == int
        assert type(dir) == int
        assert type(ena) == int
        assert type(step_delay) in [float, int]
        
        self.pul = pul
        self.dir = dir
        self.ena = ena
        self.step_delay = step_delay
        
        GPIO.setup(self.pul, GPIO.OUT)
        GPIO.setup(self.dir, GPIO.OUT)
        GPIO.setup(self.ena, GPIO.OUT)
        
        self._position = 0


    def _pulse(self):
        """
        Sends a pulse signal to the stepper motor driver.
        """
        GPIO.output(self.pul, GPIO.HIGH)
        sleep(self.step_delay/2)
        GPIO.output(self.pul, GPIO.LOW)
        sleep(self.step_delay/2)
        
        
    @property
    def position(self):
        """
        int: The current position of the stepper motor.
        """
        return self._position
    
    
    def sethome(self):
        """
        Sets the current position as the home position.
        """
        self._position = 0
        logging.info(f"Home position set. Current position: {self._position}")
        
        
    def home(self):
        """
        Moves the stepper motor to the home position.
        """
        self.step(-self.position)
        logging.info("Homed.")
        self.sethome()
        
        
    def step(self, steps):
        """
        Moves the stepper motor by the specified number of steps.

        Args:
            steps (int): The number of steps to move the stepper motor.
        """
        assert type(steps) == int
        
        GPIO.output(self.dir, GPIO.HIGH if steps > 0 else GPIO.LOW)
        GPIO.output(self.ena, GPIO.LOW)
        
        sleep(self.step_delay)
        
        for _ in range(abs(steps)):
            self._pulse()
            self._position += 1 if steps > 0 else -1

        
        GPIO.output(self.ena, GPIO.HIGH)
        logging.info(f"Moved {steps} steps. Current position: {self._position}")
        
        
    def hold(self):
        """
        Holds the stepper motor in its current position.
        """
        GPIO.output(self.ena, GPIO.LOW)
        sleep(self.step_delay)
        logging.info("Holding.")
        
        
    def release(self):
        """
        Releases the stepper motor.
        """
        GPIO.output(self.ena, GPIO.HIGH)
        sleep(self.step_delay)
        logging.info("Released.")
        
        
    def __del__(self):
        GPIO.cleanup()
        logging.info("GPIO cleaned up.")


from setuptools import setup, find_packages


setup(
    name='ncuphy',
    version='0.1.11',
    author='Kah Seng Phay',
    author_email='phay_ks@icloud.com',
    description='A Raspberry Pi Python project for educational use.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/yourusername/ncuphy',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
    ],
    python_requires='>=3.6',
    install_requires=[
        'RPi.GPIO',
        'numpy',
        'smbus2',
    ],
)
